from typing import Any, Callable, Dict, Iterable, Optional, Union

from django.contrib.admin.options import BaseModelAdmin
from django.core.exceptions import PermissionDenied
from django.http import HttpRequest, HttpResponse
from unfold.decorators import action
from unfold.typing import ActionFunction


def custom_html_field(
        function: Optional[Callable] = None,
        *,
        permissions: Optional[Iterable[str]] = None,
        description: Optional[str] = None,
        url_path: Optional[str] = None,
        attrs: Optional[Dict[str, Any]] = None,
) -> ActionFunction:
    action
    def decorator(func: Callable) -> ActionFunction:
        def inner(
                model_admin: BaseModelAdmin,
                request: HttpRequest,
                *args: Any,
                **kwargs,
        ) -> Optional[HttpResponse]:
            if permissions:
                permission_checks = (
                    getattr(model_admin, f"has_{permission}_permission")
                    for permission in permissions
                )
                # TODO add obj parameter into has_permission method call.
                # Permissions methods have following syntax: has_<some>_permission(self, request, obj=None):
                # But obj is not examined by default in django admin and it would also require additional
                # fetch from database, therefore it is not supported yet
                if not any(
                        has_permission(request) for has_permission in permission_checks
                ):
                    raise PermissionDenied
            return func(model_admin, request, *args, **kwargs)

        if permissions is not None:
            inner.allowed_permissions = permissions
        if description is not None:
            inner.short_description = description
        if url_path is not None:
            inner.url_path = url_path
        inner.attrs = attrs or {}
        inner.custom_html_field = True
        return inner

    if function is None:
        return decorator
    else:
        return decorator(function)




from django.core.management.base import BaseCommand
from django.urls import URLResolver, URLPattern


def list_urls(urlpatterns, prefix='', depth=0):
    for pattern in urlpatterns:
        if isinstance(pattern, URLPattern):
            print(f"{'  ' * depth}{prefix}{pattern.pattern}   (name={pattern.name})")
        elif isinstance(pattern, URLResolver):
            list_urls(
                pattern.url_patterns,
                prefix + pattern.pattern.regex.pattern,
                depth + 1
            )


class Command(BaseCommand):
    help = 'List all URL patterns in the Django project'

    def handle(self, *args, **options):
        from django.urls import get_resolver
        resolver = get_resolver()
        list_urls(resolver.url_patterns)



from typing import Optional, Dict, Any

from django.http import HttpRequest
from django.template.response import TemplateResponse
from unfold.sites import UnfoldAdminSite

from .forms import LoginForm

from .urls import main_admin_urlpatterns


class SuperAppAdminSite(UnfoldAdminSite):
    login_form = LoginForm

    def get_urls(self):
        urlpatterns = [
                          x for x in main_admin_urlpatterns
                      ] + super().get_urls()

        return urlpatterns

    def custom_admin_page(self, view):
        def generator(request: HttpRequest, extra_context: Optional[Dict[str, Any]] = None,
                      **kwargs) -> TemplateResponse:
            app_list = self.get_app_list(request)

            context = {
                **self.each_context(request),
                "title": self.index_title,
                "subtitle": None,
                "app_list": app_list,
                "index": True,
                **(extra_context or {}),
            }

            request.current_app = self.name

            view_instance = view.as_view()
            return view_instance(request, context, **kwargs)

        return generator


superapp_admin_site = SuperAppAdminSite()



import os
import click
import requests
from copier import run_copy, run_update
from pydantic import ValidationError

from django_superapp.types import SuperAppTemplate


def get_superapp_templates() -> SuperAppTemplate:
    url = "https://raw.githubusercontent.com/django-superapp/django-superapp/main/superapp_templates.json"
    response = requests.get(url)
    response.raise_for_status()
    try:
        return SuperAppTemplate.parse_obj(response.json())
    except ValidationError as e:
        print("Error parsing JSON:", e)
        raise


superapp_templates = get_superapp_templates()


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    '--project-template',
    type=click.Choice([
        project_name for project_name in superapp_templates.projects.keys()
    ], case_sensitive=False),
    default=[
        project_name for project_name in superapp_templates.projects.keys()
        if superapp_templates.projects[project_name].default
    ][0],
    show_default=True,
    show_choices=True,
    prompt=True,
)
@click.argument('target_directory')
def create_project(project_template, target_directory):
    """Bootstrap a project into target directory."""

    # Get the chosen project details
    project_template_obj = superapp_templates.projects[project_template]

    run_copy(
        project_template_obj.repo,
        str(target_directory),
        vcs_ref=project_template_obj.branch,
        data={
            'project_template': project_template,
            'project_template_obj': project_template_obj,
        },
        **(project_template_obj.kwargs or {}),
    )


@cli.command()
@click.argument('target_directory')
def update_project(target_directory):
    """Update a project with remote changes."""

    run_update(
        str(target_directory),
        overwrite=True,
    )


@cli.command()
@click.option(
    '--app-template',
    type=click.Choice([
        slug for slug in superapp_templates.apps.keys()
    ]),
    help='Name of the application template to use',
    show_choices=True,
    prompt=True,

)
def create_app(app_template):
    """Create an app inside the project."""
    target_path = os.path.join("superapp", "apps", app_template)

    if not os.path.exists("superapp/apps"):
        raise click.ClickException(
            "The superapp/apps directory does not exist. Make sure you are in the root of the project."
        )

    # Get the chosen project details
    app_template = superapp_templates.apps[app_template]

    run_copy(
        app_template.repo,
        str(target_path),
        vcs_ref=app_template.branch,
        **(app_template.kwargs or {}),
    )


@cli.command()
@click.argument('target_directory')
def update_app(target_directory):
    """Update a project with remote changes."""

    run_update(
        str(target_directory),
        overwrite=True,
    )

if __name__ == '__main__':
    cli()



from dataclasses import dataclass
from typing import Dict, Optional

from pydantic import BaseModel


@dataclass
class SuperAppProjectTemplate:
    name: str
    description: str
    repo: str
    branch: str
    kwargs: Optional[Dict] = None
    default: Optional[bool] = False


@dataclass
class SuperAppAppTemplate:
    name: str
    description: str
    repo: str
    branch: str
    kwargs: Optional[Dict] = None


@dataclass
class SuperAppTemplate(BaseModel):
    projects: Dict[str, SuperAppProjectTemplate]
    apps: Dict[str, SuperAppAppTemplate]
